@@ -79,71 +79,73 @@ int dialog_menu(const char *title, const char *prompt, int height, int width,
 	int key = 0, button = GTK_RESPONSE_OK, choice = 0, scroll = 0, max_choice;
 	WINDOW *dialog, *menu;
 
- 	max_choice = MIN(menu_height, item_no);
-
-	if (gnome_mode)	{
+	max_choice = MIN(menu_height, item_no);
+	
+	if(gnome_mode) 	{
 		GtkWidget *w;
-		GtkWidget *but;
+		GtkWidget *options;
 		GtkWidget *first_button;
-		GtkWidget *butbox;
-		GtkWidget *sw, *viewport;
- 
+		GtkWidget *but;
+		GtkWidget *menu;
+
 		w = gtk_dialog_new_with_buttons (title,	NULL,
 				GTK_DIALOG_DESTROY_WITH_PARENT,
 				GTK_STOCK_CANCEL,
 				GTK_RESPONSE_CANCEL,
-				NULL);		
+				NULL);
                 gtk_signal_connect (GTK_OBJECT (w), "response",
                               GTK_SIGNAL_FUNC (esc_cancel), NULL);
 		gtk_window_set_position (GTK_WINDOW (w), GTK_WIN_POS_CENTER);
-		
-		label_autowrap (GTK_DIALOG(w)->vbox, prompt, width);
+
+		label_autowrap(GTK_DIALOG(w)->vbox, prompt, width);
 
 		/*
-		 * Setup the containers.
+		 * Setup the option menu
 		 */
-		sw = gtk_scrolled_window_new (NULL, NULL);
- 		gtk_box_pack_start_defaults (
-			GTK_BOX (GTK_DIALOG (w)->vbox), sw);
-		gtk_widget_show (sw);
+		options = gtk_option_menu_new ();
+ 		gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (w)->vbox), options);
+		gtk_widget_show (options);
 
- 		viewport = gtk_viewport_new (NULL, NULL);
-		gtk_container_add (GTK_CONTAINER (sw), viewport);
- 		gtk_widget_show (viewport);
- 
- 		butbox = gtk_vbox_new (FALSE, 0);
- 		gtk_container_add (GTK_CONTAINER (viewport), butbox);
- 		gtk_widget_show (butbox);
-
-		gtk_container_set_border_width (GTK_CONTAINER (butbox),
-						GNOME_PAD);
 		/*
 		 * Add the buttons.
 		 */
 		the_items = items;
 		first_button = NULL;
-		
-		for(i=0; i< max_choice; i++)
+		menu = gtk_option_menu_get_menu (options);
+		if (!GTK_IS_MENU (menu)) {
+			menu = gtk_menu_new ();
+			gtk_option_menu_set_menu (GTK_OPTION_MENU (options), menu);
+			gtk_widget_show (menu);
+		}
+
+		for(i=0; i< item_no; i++)
 		{
 			char *x = (char *)items[2*i];
 			char *y = (char *)items[2*i+1];
 			char *p;
 
 			p = g_strdup_printf("%s  -   %s", x, y);
-			but=gtk_button_new_with_label(p);
+			but = gtk_menu_item_new_with_label (p);
 			g_free(p);
+			gtk_widget_show (but);
 
 			if (first_button == NULL)
 				first_button = but;
+			else
+				gtk_signal_connect(GTK_OBJECT(but), "activate",
+						   GTK_SIGNAL_FUNC(okayed), GUINT_TO_POINTER(i));
+
+			gtk_menu_shell_append (GTK_MENU_SHELL (menu), but);
 
-			gtk_box_pack_start(GTK_BOX(butbox), but, TRUE, TRUE, 0);
-			gtk_signal_connect(GTK_OBJECT(but), "clicked",
-			GTK_SIGNAL_FUNC(okayed), GUINT_TO_POINTER(i));
 		}
 
+
 		gtk_widget_show_all(w);
-		if (first_button != NULL)
-			gtk_widget_grab_focus (first_button);
+		if (first_button != NULL) {
+			gtk_menu_shell_activate_item (GTK_MENU_SHELL (menu), first_button, FALSE);
+			gtk_signal_connect(GTK_OBJECT(first_button), "activate",
+					   GTK_SIGNAL_FUNC(okayed), GUINT_TO_POINTER(0));
+		}
 		gtk_main();
 		return 0;
 	}
@@ -201,7 +203,7 @@ int dialog_menu(const char *title, const char *prompt, int height, int width,
 	tag_x = 0;
 	item_x = 0;
 	/* Find length of longest item in order to center menu */
-	for (i = 0; i < max_choice; i++) {
+	for (i = 0; i < item_no; i++) {
 		tag_x = MAX(tag_x,
 		    strlen(items[i * 2]) + strlen(items[i * 2 + 1]) + 2);
 		item_x = MAX(item_x, strlen(items[i * 2]));
@@ -210,7 +212,7 @@ int dialog_menu(const char *title, const char *prompt, int height, int width,
 	item_x = tag_x + item_x + 2;
 
 	/* Print the menu */
-	for (i = 0; i < item_no; i++)
+	for (i = 0; i < max_choice; i++)
 		print_item(menu, items[i * 2], items[i * 2 + 1],
 			   i, i == choice);
 	wnoutrefresh(menu);
